'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };
/* eslint no-underscore-dangle: 0 */
/* eslint no-param-reassign: 0 */
/* eslint no-use-before-define: 0 */

var _denormalizr = require('denormalizr');

function mergeMappings(entity, mapObj) {
  if (!mapObj) {
    return entity;
  }

  if (typeof mapObj === 'function') {
    return mapObj(entity);
  }

  if ((typeof entity === 'undefined' ? 'undefined' : _typeof(entity)) === 'object') {
    if (!Array.isArray(entity) && Object.keys(mapObj).includes('id')) {
      return Object.assign({}, entity, mapObj);
    }

    return Object.assign({}, mapObj[entity.id], entity);
  }

  return entity;
}

function iterateOverObject(object, mappings) {
  if (!object || mappings === {}) return object;
  var mappingsKeys = Object.keys(mappings || {});

  return Object.keys(object).reduce(function (newObject, key) {
    var isArray = Array.isArray(newObject[key]);

    if (mappingsKeys.includes(key)) {
      if (isArray) {
        newObject[key] = newObject[key].map(function (item) {
          return mergeMappings(item, mappings[key]);
        });
      } else {
        newObject[key] = mergeMappings(newObject[key], mappings[key]);
      }
    }

    if (isArray || _typeof(newObject[key]) === 'object') {
      newObject[key] = iterateOverObject(newObject[key], mappings);
    }

    return newObject;
  }, object);
}

exports.default = function (entity, entities, schema, mappings) {
  if (entity === null) return null;

  // entity ~ 1 or "abc"
  if (typeof entity === 'number' || typeof entity === 'string') {
    var state = (0, _denormalizr.denormalize)(entity, entities, schema);
    state = iterateOverObject(state, mappings);
    return mappings ? mergeMappings(state, mappings[schema._key]) : state;
  } else if ((typeof entity === 'undefined' ? 'undefined' : _typeof(entity)) === 'object') {
    //
    // entity ~ [1, "abc"]
    if (Array.isArray(entity)) {
      if (typeof entity[0] === 'number' || typeof entity[0] === 'string') {
        var _state = (0, _denormalizr.denormalize)(entity, entities, schema);
        _state = iterateOverObject(_state, mappings);

        if (mappings && schema._itemSchema._key in mappings) {
          return _state.map(function (item) {
            return mergeMappings(item, mappings[schema._itemSchema._key]);
          });
        }

        return _state.map(function (item) {
          if (!item || (typeof item === 'undefined' ? 'undefined' : _typeof(item)) !== 'object') return null;
          item = mergeMappings(item, entity.find(function (i) {
            return i === item.id;
          }));
          return mappings ? mergeMappings(item, mappings[schema._itemSchema._key]) : item;
        });
      }
    } else {
      var entityKeys = Object.keys(entity);

      // entity ~ { id: 1, isLoading: false }
      if (entityKeys.includes('id')) {
        var _state3 = (0, _denormalizr.denormalize)(entity.id, entities, schema);
        _state3 = iterateOverObject(_state3, mappings);
        _state3 = mergeMappings(_state3, entity);
        return mappings ? mergeMappings(_state3, mappings[schema._key]) : _state3;
      }

      // entity ~ { 1: { isLoading: false } }
      var _state2 = (0, _denormalizr.denormalize)(entityKeys.map(function (id) {
        return id;
      }), entities, schema);
      _state2 = iterateOverObject(_state2, mappings);
      return _state2.map(function (item) {
        if (!item || !item.id) {
          return null;
        }
        return mergeMappings(entity[item.id], item);
      });
    }
  }

  // fall back to plain old denormalize
  return (0, _denormalizr.denormalize)(entity, entities, schema);
};